#include "tsqlparse.h"

#include <iostream>
#include <QFile>


int main (int argc, char *argv[])
{
	QString fileName;
	
	if (argc < 2 || argv[1] == NULL)
	{
		std::cout << "Usage: " << argv[0] << " <input>.sql" << std::endl;
		return 1;
	}
	else
	{
		fileName = argv[1];
	}
	
	try
	{

		QFile file(fileName);
		if (!file.open(QFile::ReadOnly | QFile::Text))
			return 1;

		QString statement(file.readAll());
		if(statement.lastIndexOf(';') == -1)
		{
			statement += ';';
		}

		QRegExp reg("\\s+");
		QStringList sl = statement.split(reg, QString::SkipEmptyParts);
		QString const& firstWord = sl.first();

		if( QString::compare("CALL", firstWord, Qt::CaseInsensitive)      == 0 ||
		    QString::compare("ANALYZE", firstWord, Qt::CaseInsensitive)   == 0 ||			    
		    QString::compare("DECLARE", firstWord, Qt::CaseInsensitive)   == 0 ||
		    QString::compare("BEGIN", firstWord, Qt::CaseInsensitive)     == 0 ||
		    QString::compare("CREATE", firstWord, Qt::CaseInsensitive)    == 0 ||
		    QString::compare("ALTER", firstWord, Qt::CaseInsensitive)     == 0 ||
		    QString::compare("LOCK", firstWord, Qt::CaseInsensitive)      == 0 ||
		    QString::compare("EXPLAIN", firstWord, Qt::CaseInsensitive)   == 0 ||			    
		    QString::compare("DROP", firstWord, Qt::CaseInsensitive)      == 0 ||
		    QString::compare("COMMIT", firstWord, Qt::CaseInsensitive)     == 0 ||
		    QString::compare("ROLLBACK", firstWord, Qt::CaseInsensitive)   == 0 ||
		    QString::compare("GRANT", firstWord, Qt::CaseInsensitive)     == 0 ||
		    QString::compare("TRUNCATE", firstWord, Qt::CaseInsensitive)  == 0 ||
		    QString::compare("SAVEPOINT", firstWord, Qt::CaseInsensitive) == 0 ||
		    QString::compare("SET", firstWord, Qt::CaseInsensitive) == 0
			)
		{
			std::cout << "PLSQL:" << std::endl;
			std::auto_ptr <SQLParser::Statement> stat = StatementFactTwoParmSing::Instance().create("OraclePLSQL", statement, fileName);
			std::cout << stat->root()->toStringRecursive().toStdString() << std::endl;
			return 0;
		} else if( QString::compare("WITH", firstWord, Qt::CaseInsensitive)    == 0 ||
			   QString::compare("SELECT", firstWord, Qt::CaseInsensitive) == 0 ||
			   QString::compare("INSERT", firstWord, Qt::CaseInsensitive) == 0 ||
			   QString::compare("UPDATE", firstWord, Qt::CaseInsensitive)  == 0 ||
			   QString::compare("DELETE", firstWord, Qt::CaseInsensitive) == 0 ||
			   QString::compare("MERGE", firstWord, Qt::CaseInsensitive) == 0
			) {
			std::cout << "SQL:" << std::endl;				
			std::auto_ptr <SQLParser::Statement> stat = StatementFactTwoParmSing::Instance().create("OracleSQL", statement, fileName);
			std::cout << stat->root()->toStringRecursive().toStdString() << std::endl;
			return 0;
		} else {
			std::cout << "Unknown:" << firstWord.QString::toStdString() << std::endl;
			return 1;
		}
			

	} catch(SQLParser::ParseException const &e)
	{
		std::cout << "An exception caught" << std::endl;
		return 1;
	}

	return 0;
};

