WITH
	CODES2CODELOCALES AS
	(
		SELECT T6.CDL_NAME AS COD_NAME, T7.CDL_NAME AS COD_COD_NAME, T14.COD_OID
		FROM SERVICEDESK.ITSM_CODES T14
		LEFT OUTER JOIN SERVICEDESK.ITSM_CODES_LOCALE T6 on (T6.CDL_COD_OID=T14.COD_OID)
		LEFT OUTER JOIN SERVICEDESK.ITSM_CODES_LOCALE T7 on (T7.CDL_COD_OID=T14.COD_COD_OID)
	)
	, INCIDENT AS
	(
		SELECT T1.*,
		       C2CL1.COD_NAME as "Closure Code", C2CL1.COD_COD_NAME as "Closure Code Parent",
		       C2CL2.COD_NAME as "Reason Caused Code", C2CL2.COD_COD_NAME as "Reason Caused Code Parent",
		       T11.CDL_NAME "Severity", T13.CDL_NAME "Business Impact", T16.CDL_NAME "Priority",
		       T2.RCT_NAME "Status", T12.RCT_NAME "Category", T99.RCT_NAME "Folder"
		FROM SERVICEDESK.ITSM_INCIDENTS T1
		JOIN SERVICEDESK.ITSM_CODES_LOCALE T11   on (T1.INC_SEV_OID=T11.CDL_COD_OID)
		JOIN SERVICEDESK.ITSM_CODES_LOCALE T13   on (T1.INC_IMP_OID=T13.CDL_COD_OID)
		JOIN SERVICEDESK.ITSM_CODES_LOCALE T16   on (T1.INC_PRI_OID=T16.CDL_COD_OID)
		JOIN SERVICEDESKREPO.REP_CODES_TEXT T2   on (T1.INC_STA_OID=T2.RCT_RCD_OID)
		JOIN SERVICEDESKREPO.REP_CODES_TEXT T12  on (T1.INC_CAT_OID=T12.RCT_RCD_OID)
		JOIN SERVICEDESKREPO.REP_CODES_TEXT T99  on (T1.INC_POO_OID=T99.RCT_RCD_OID)
		LEFT OUTER JOIN CODES2CODELOCALES C2CL1  on (T1.INC_CLO_OID=C2CL1.COD_OID)
		LEFT OUTER JOIN CODES2CODELOCALES C2CL2  on (T1.INC_CLA_OID=C2CL2.COD_OID) 
		WHERE T1."REG_CREATED" between SYSDATE-1 and SYSDATE
	)
	, WORKGROUPHISTORY AS
	(
		SELECT I.INC_ID
		, MAX(T101.HIN_SUBJECT) KEEP (DENSE_RANK FIRST ORDER BY (T101.REG_CREATED)) AS "First"
		, MAX(T101.HIN_SUBJECT) KEEP (DENSE_RANK LAST ORDER BY (T101.REG_CREATED)) AS "Last"
		FROM
		SERVICEDESK.ITSM_HISTORYLINES_INCIDENT T101
		JOIN INCIDENT I on (T101.HIN_INC_OID = I.INC_OID)
--		FROM SERVICEDESK.ITSM_INCIDENTS I (T101.HIN_INC_OID = I.INC_OID)
		WHERE T101.HIN_SUBJECT like 'To Workgroup from%'
--		AND I."REG_CREATED" between SYSDATE-1 and SYSDATE		
		GROUP BY I.INC_ID
	)
SELECT
  INCIDENT.INC_ID "ID"
 ,INCIDENT."Status"
 ,INCIDENT.INC_DESCRIPTION "Description"
 ,T4.WOG_SEARCHCODE "Workgroup"
 ,T5.PER_SEARCHCODE "Person"
 ,INCIDENT.INC_SOLUTION "Solution"
 ,INCIDENT."Closure Code"
 ,INCIDENT."Closure Code Parent"
 ,INCIDENT."Reason Caused Code"
 ,INCIDENT."Reason Caused Code Parent"
 ,T10.CIT_SEARCHCODE "CI"
 ,INCIDENT."Severity"
 ,INCIDENT."Category"
 ,INCIDENT."Business Impact"
 ,INCIDENT."Priority"
 ,TO_CHAR(INCIDENT."REG_CREATED", 'DD-MM-YY HH24:MI:SS') "Registered"
 ,TO_CHAR(INCIDENT."INC_DEADLINE", 'DD-MM-YY HH24:MI:SS') "Deadline"
 ,TO_CHAR(INCIDENT."INC_ACTUALFINISH", 'DD-MM-YY HH24:MI:SS') "Finish"
 ,T3.ICF_INCSHORTTEXT3 "Message Group"
 ,T3.ICF_INCSHORTTEXT4 "Application"
 ,T3.ICF_INCSHORTTEXT2 "Msg ID"
 ,INCIDENT."Folder"
 ,WORKGROUPHISTORY."First" "First WG"
 ,WORKGROUPHISTORY."Last" "Last WG"
 ,T102.HIN_SUBJECT "Frirst Pri"
FROM INCIDENT
JOIN SERVICEDESK.ITSM_INC_CUSTOM_FIELDS T3  ON (INCIDENT.INC_OID=T3.ICF_INC_OID)
JOIN SERVICEDESK.ITSM_WORKGROUPS T4         ON (INCIDENT.INC_ASSIGN_WORKGROUP=T4.WOG_OID)
JOIN WORKGROUPHISTORY                       ON (INCIDENT.INC_ID = WORKGROUPHISTORY.INC_ID)
LEFT OUTER JOIN SERVICEDESK.ITSM_PERSONS T5 ON (INCIDENT.INC_ASSIGN_PERSON_TO=T5.PER_OID)
LEFT OUTER JOIN SERVICEDESK.ITSM_CONFIGURATION_ITEMS T10 on (INCIDENT.INC_CIT_OID=T10.CIT_OID)
LEFT OUTER JOIN SERVICEDESK.ITSM_HISTORYLINES_INCIDENT T102 on (INCIDENT.INC_OID = T102.HIN_INC_OID and T102.HIN_SUBJECT like 'Priority set to%')
;
