WITH TIMEGROUPED_RAWDATA
AS
(
	SELECT /*+ LEADING(SN DI MD SH OT) CARDINALITY(OT 1000) */
	SH.METRIC_ID AS METRIC_ID ,
	OT.BSLN_GUID AS BSLN_GUID ,
	OT.TIMEGROUP AS TIMEGROUP ,
	SH.VALUE AS OBS_VALUE
	FROM
	DBA_HIST_SNAPSHOT SN ,
	DBA_HIST_DATABASE_INSTANCE DI ,
	SYS.WRH$_SYSMETRIC_HISTORY SH ,
	BSLN_METRIC_DEFAULTS MD ,
	TABLE(:B1 ) OT
	WHERE SN.DBID = :B6
	AND SN.SNAP_ID BETWEEN :B5 AND :B4
	AND DI.DBID = SN.DBID
	AND DI.INSTANCE_NUMBER = SN.INSTANCE_NUMBER
	AND DI.STARTUP_TIME = SN.STARTUP_TIME
	AND DI.INSTANCE_NAME = :B3
	AND SH.SNAP_ID = SN.SNAP_ID
	AND SH.DBID = SN.DBID
	AND SH.INSTANCE_NUMBER = SN.INSTANCE_NUMBER
	AND SH.GROUP_ID = 2
	AND SH.METRIC_ID = MD.METRIC_ID
	AND MD.STATUS = :B2
	AND OT.OBS_TIME = TRUNC(SH.END_TIME, 'HH24')
)
(
SELECT
	BSLN_STATISTICS_T
	(
		BSLN_GUID ,METRIC_ID ,:B11 ,:B10 ,TIMEGROUP ,SAMPLE_COUNT ,AVERAGE ,MINIMUM ,MAXIMUM ,SDEV ,
		PCTILE_25 ,PCTILE_50 ,PCTILE_75 ,PCTILE_90 ,PCTILE_95 ,PCTILE_99 ,EST_SAMPLE_COUNT ,EST_SLOPE ,EST_INTERCEPT ,
		CASE WHEN EST_SLOPE = 0 THEN 0 ELSE GREATEST(0,NVL(100-(25*POWER((1-EST_MU1/EST_SLOPE), 2)*(EST_SAMPLE_COUNT-1) ),0)) END ,
		LN( 1000) * EST_SLOPE + EST_INTERCEPT ,
		LN(10000) * EST_SLOPE + EST_INTERCEPT
	)
	FROM
	(
		SELECT METRIC_ID ,BSLN_GUID ,TIMEGROUP ,EST_MU AS EST_SLOPE ,EST_MU * LN(ALPHA) + X_M AS EST_INTERCEPT ,TO_NUMBER(NULL) AS EST_FIT_QUALITY ,
		CASE WHEN COUNT_BELOW_X_J > 0 THEN (SUM_BELOW_X_J + (N-M+1)*(X_J-X_M))/COUNT_BELOW_X_J - X_J ELSE TO_NUMBER(NULL) END AS EST_MU1 ,
		EST_SAMPLE_COUNT ,N AS SAMPLE_COUNT ,AVERAGE ,MINIMUM ,MAXIMUM ,SDEV ,PCTILE_25 ,PCTILE_50 ,PCTILE_75 ,PCTILE_90 ,PCTILE_95 ,PCTILE_99
		FROM
		(
			SELECT METRIC_ID ,BSLN_GUID ,TIMEGROUP ,MAX(N) AS N ,COUNT(RRANK) AS EST_SAMPLE_COUNT ,
			CASE WHEN COUNT(RRANK) > 3 THEN ( SUM(OBS_VALUE) + ( MAX(N) - MAX(RRANK) ) * MAX(OBS_VALUE) - (MAX(N) - MIN(RRANK) + 1) * MIN(OBS_VALUE) ) / (COUNT(RRANK)-1)
			ELSE TO_NUMBER(NULL) END AS EST_MU ,
			(MAX(N) - MIN(RRANK) + 1) / (MAX(N) + 1) AS ALPHA ,
			MIN(OBS_VALUE) AS X_M ,MAX(OBS_VALUE) AS X_L ,MAX(RRANK) AS L ,MIN(RRANK) AS M ,MAX(MID_TAIL_VALUE) AS X_J ,
			SUM(CASE WHEN OBS_VALUE < MID_TAIL_VALUE THEN OBS_VALUE ELSE 0 END ) AS SUM_BELOW_X_J ,
			SUM(CASE WHEN CUME_DIST < :B7 THEN 1 ELSE 0 END ) AS COUNT_BELOW_X_J ,
			MAX(MAX_VAL) AS MAXIMUM ,MAX(MIN_VAL) AS MINIMUM ,MAX(AVG_VAL) AS AVERAGE ,MAX(SDEV_VAL) AS SDEV ,MAX(PCTILE_25) AS PCTILE_25 ,MAX(PCTILE_50) AS PCTILE_50 ,
			MAX(PCTILE_75) AS PCTILE_75 ,MAX(PCTILE_90) AS PCTILE_90 ,MAX(PCTILE_95) AS PCTILE_95 ,MAX(PCTILE_99) AS PCTILE_99
			FROM
			(
				SELECT METRIC_ID ,BSLN_GUID ,TIMEGROUP ,OBS_VALUE AS OBS_VALUE,
				CUME_DIST () OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ORDER BY OBS_VALUE ) AS CUME_DIST ,
				COUNT(1) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ) AS N ,
				ROW_NUMBER () OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ORDER BY OBS_VALUE) AS RRANK ,
				PERCENTILE_DISC(:B7 ) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS MID_TAIL_VALUE ,
				MAX(OBS_VALUE) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ) AS MAX_VAL ,
				MIN(OBS_VALUE) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ) AS MIN_VAL ,
				AVG(OBS_VALUE) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ) AS AVG_VAL ,
				STDDEV(OBS_VALUE) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP ) AS SDEV_VAL ,
				PERCENTILE_CONT(0.25) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS PCTILE_25 ,
				PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS PCTILE_50 ,
				PERCENTILE_CONT(0.75) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS PCTILE_75 ,
				PERCENTILE_CONT(0.90) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS PCTILE_90 ,
				PERCENTILE_CONT(0.95) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS PCTILE_95 ,
				PERCENTILE_CONT(0.99) WITHIN GROUP (ORDER BY OBS_VALUE ASC) OVER (PARTITION BY METRIC_ID, BSLN_GUID, TIMEGROUP) AS PCTILE_99
				FROM TIMEGROUPED_RAWDATA D
			) X
			WHERE X.CUME_DIST >= :B9 AND X.CUME_DIST <= :B8
			GROUP BY METRIC_ID ,BSLN_GUID ,TIMEGROUP
		)
	)
)
;